# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mplwidgets.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MatplotlibMainWindow(object):
    def setupUi(self, MatplotlibMainWindow):
        MatplotlibMainWindow.setObjectName("MatplotlibMainWindow")
        MatplotlibMainWindow.resize(800, 600)
        self.central_widget = QtWidgets.QWidget(MatplotlibMainWindow)
        self.central_widget.setObjectName("central_widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.central_widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.matplotlib_widget = PMMatplotlibQt5Widget(self.central_widget)
        self.matplotlib_widget.setObjectName("matplotlib_widget")
        self.horizontalLayout.addWidget(self.matplotlib_widget)
        MatplotlibMainWindow.setCentralWidget(self.central_widget)
        self.statusbar = QtWidgets.QStatusBar(MatplotlibMainWindow)
        self.statusbar.setObjectName("statusbar")
        MatplotlibMainWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MatplotlibMainWindow)
        self.toolBar.setMovable(False)
        self.toolBar.setFloatable(False)
        self.toolBar.setObjectName("toolBar")
        MatplotlibMainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.dockWidget = QtWidgets.QDockWidget(MatplotlibMainWindow)
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.dockWidgetContents_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.mpl_settings_panel = SettingsPanel(self.dockWidgetContents_2)
        self.mpl_settings_panel.setObjectName("mpl_settings_panel")
        self.verticalLayout_2.addWidget(self.mpl_settings_panel)
        self.dockWidget.setWidget(self.dockWidgetContents_2)
        MatplotlibMainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.dockWidget)
        self.dockWidget.raise_()

        self.retranslateUi(MatplotlibMainWindow)
        QtCore.QMetaObject.connectSlotsByName(MatplotlibMainWindow)

    def retranslateUi(self, MatplotlibMainWindow):
        _translate = QtCore.QCoreApplication.translate
        MatplotlibMainWindow.setWindowTitle(_translate("MatplotlibMainWindow", "MainWindow"))
        self.toolBar.setWindowTitle(_translate("MatplotlibMainWindow", "toolBar"))
from pmgwidgets import PMMatplotlibQt5Widget, SettingsPanel
