#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2020/1/6 15:41
# @Author  : hzy
# @Site    :
# @File    : setup.py
# @Software: PyCharm
# @Description:

import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name = "pmgwidgets",
    version = "0.1.11",
    author = "Hou Zhanyi",
    description = "General widgets for pyminer",
    long_description = long_description,
    long_description_content_type="text/markdown",
    url = "",
    packages = setuptools.find_packages(),
    classifiers=[],
    resources=['pmgwidgets/treeviews/translations']


)
